package ch.zhaw.ads;

import java.util.Arrays;
import java.util.Random;
import java.util.function.Consumer;

public class QuickerSortServer implements CommandExecutor {
    int QUICKSORT_THRESHOLD = 70;

    @Override
    public String execute(String command) {
        StringBuffer result = new StringBuffer();

        int[] testdata = new int[1000];
        Random random = new Random();
        for (int i=0; i < testdata.length; i++) {
            testdata[i] = random.nextInt(1000000);
        }

        double arraysSortRuntime = timeit(Arrays::sort, testdata);
        result.append("Is sorted: " + checkSorted(testdata) + "\n");
        result.append("quickSort runtime: " + arraysSortRuntime + "ms\n");
        result.append("quickSort runtime:\n");
        // for (int i = 0; i < 1000; i++)
        for (int i = 40; i < 100; i++) {
            QUICKSORT_THRESHOLD = i;
            double quickSortRuntime = timeit(this::quickSort, testdata);
            result.append(QUICKSORT_THRESHOLD + "," + quickSortRuntime + "\n");
        }
        return result.toString();
    }

    private void quickSort(int[] a, int left, int right) {
        if (left < right) {
            int mid = partition (a, left, right);
            if (mid <= QUICKSORT_THRESHOLD) {
                insertSort(a);
            } else {
                quickSort(a, left, mid-1);
                quickSort(a, mid , right);
            }
        }
    }

    private void quickSort(int[] a){
        quickSort(a, 0, a.length-1);
    }

    private static int partition (int[] a, int left, int right) {
        int pivot = a[(left + right) / 2];
        while (left <= right) {
            while (a[left] < pivot) {
                left++;
            }
            while (a[right] > pivot) {
                right--;
            }
            if (left <= right) {
                int temp = a[left];
                a[left] = a[right];
                a[right] = temp;
                left++;
                right--;
            }
        }
        return left;
    }

    private boolean checkSorted(int[] a) {
        for (int i=1; i < a.length; i++) {
            if (a[i-1] > a[i]) return false;
        }
        return true;
    }

    private void insertSort(int[] a) {
        for (int k = 1; k < a.length; k++)
            if (a[k] < a[k-1]){
                int x = a[k];
                int i;
                for (i = k; ((i > 0) && (a[i-1] > x));i--)
                    a[i] = a[i-1];
                a[i] = x;
            }
    }

    private double timeit(Consumer<int[]>func, int[] a) {
        long end, start = System.currentTimeMillis();
        int count = 0;
        int[] testarray = new int[a.length];
        do {
            System.arraycopy(a, 0, testarray, 0, a.length);
            func.accept(testarray);
            count++;
            end = System.currentTimeMillis();
        } while (end - start < 1000);
        return (double)(end-start)/count;
    }
}
